%% BuildLookupG.m
%
% Build the lookup table for partial derivatives of G.
%
% SEE: G.m
%
% Matthieu Guerquin-Kern, Biomedical Imaging Group / EPF Lausanne,
% 31-10-2009 (dd-mm-yyyy)

function lookup = BuildLookupG(D)

if exist('lookupG.mat','file')
    load lookupG.mat;
else
    lookup = [];
end

if length(lookup)>=D+1
    tmp = lookup;
    lookup = cell(1,D+1);
    for d=1:D+1
        lookup{d} = tmp{d};
    end
    return;
end

if isempty(lookup)
    lookup{1}{1}.n = 1;
    lookup{1}{1}.o = [0;0];
    lookup{1}{1}.coeff = 1;
end

D_cur = length(lookup);
tmp = lookup;
lookup = cell(1,D+1);
for d=1:D_cur
    lookup{d} = tmp{d};
end

for d = D_cur:D
    lookup{d+1} = cell(1,d+1); % order of derivative wrt 2nd variable
    % the order wrt the second variable follows
    for d2 = 0:floor(d/2) % order of derivative wrt 2nd variable
        lookup{d+1}{d2+1}.n = [];
        lookup{d+1}{d2+1}.o = [];
        lookup{d+1}{d2+1}.coeff = [];
        for n = 1:length(lookup{d}{d2+1}.n)
            if lookup{d}{d2+1}.o(1,n) > 0
                lookup{d+1}{d2+1}.n     = [lookup{d+1}{d2+1}.n,     lookup{d}{d2+1}.n(n)];
                lookup{d+1}{d2+1}.o     = [lookup{d+1}{d2+1}.o,     lookup{d}{d2+1}.o(:,n)-[1;0]];
                lookup{d+1}{d2+1}.coeff = [lookup{d+1}{d2+1}.coeff, lookup{d}{d2+1}.coeff(n)*lookup{d}{d2+1}.o(1,n)];
            end
            lookup{d+1}{d2+1}.n     = [lookup{d+1}{d2+1}.n,     lookup{d}{d2+1}.n(n) + 1        ];
            lookup{d+1}{d2+1}.o     = [lookup{d+1}{d2+1}.o,     lookup{d}{d2+1}.o(:,n) + [1;0]  ];
            lookup{d+1}{d2+1}.coeff = [lookup{d+1}{d2+1}.coeff, -lookup{d}{d2+1}.coeff(n)       ];
        end
        % looking for simplifications
        for n = 1:length(lookup{d+1}{d2+1}.n)
            nn = lookup{d+1}{d2+1}.n(n);
            o1n = lookup{d+1}{d2+1}.o(1,n);
            ind = find(lookup{d+1}{d2+1}.n==nn&lookup{d+1}{d2+1}.o(1,:)==o1n);
            if numel(ind)>1
                if ind(1)==n
                    lookup{d+1}{d2+1}.coeff(n) = sum(lookup{d+1}{d2+1}.coeff(ind));
                    lookup{d+1}{d2+1}.coeff(ind(2:end)) = 0;
                end
            end
        end
        ind = find(lookup{d+1}{d2+1}.coeff~=0);
        [lookup{d+1}{d2+1}.n,ind2] = sort(lookup{d+1}{d2+1}.n(ind));
        lookup{d+1}{d2+1}.o = lookup{d+1}{d2+1}.o(:,ind(ind2));
        lookup{d+1}{d2+1}.coeff = lookup{d+1}{d2+1}.coeff(ind(ind2));
        
        % symmetry
        if d2~=d/2
            lookup{d+1}{d+1-d2}.n       =   lookup{d+1}{d2+1}.n;
            lookup{d+1}{d+1-d2}.o       =   zeros(size(lookup{d+1}{d2+1}.o));
            lookup{d+1}{d+1-d2}.o(1,:)  =   lookup{d+1}{d2+1}.o(2,:);
            lookup{d+1}{d+1-d2}.o(2,:)  =   lookup{d+1}{d2+1}.o(1,:);
            lookup{d+1}{d+1-d2}.coeff   =   lookup{d+1}{d2+1}.coeff;
        end
    end
end

save('lookupG.mat','lookup');