%% MRDataEllipse.m
%
% Returns the MR data corresponding to the indicator function of an ellipse
% that is modulated by a polynomial profile, for the given kspace samples.
%
% Matthieu Guerquin-Kern, Biomedical Imaging Group / EPF Lausanne,
% 31-10-2009 (dd-mm-yyyy)

function m = MRDataEllipse(ellipse,s,k)

theta = ellipse.angle;
width = ellipse.width;
center = ellipse.center;
amplitude = ellipse.amp;
FOV = ellipse.FOV;

Rmat = [cos(theta) ,-sin(theta);sin(theta) ,cos(theta)];
Dmat = diag(width/2);

N = length(s);
D =  floor((sqrt(1+N*8)-3)/2); % works only in 2D!
Np = N; % N is sufficient for matrix inversion. However, for high degree one should consider to increase this number in order to improve conditionning
u = zeros(2,Np);
u(1,:) = rand(1,Np)-0.5;
u(2,:) = rand(1,Np)-0.5;
r = Rmat*Dmat*u+repmat(center(:),[1,Np]);%figure(1);plot(r(1,:),r(2,:),'g.');hold on;plot(u(1,:),u(2,:),'r.');hold off;
m = Polynomial2DMatrix(r,D);
mu = Polynomial2DMatrix(u,D);%cond(mu)
t = mu\m*s(:);


%% Computations

% Build the lookup table for partial derivatives
lookup = BuildLookupG(D);
m = zeros(1,size(k,2));
w = -2*pi*k;
wu = Dmat*Rmat'*w;
modw = sqrt(sum(wu.^2,1));
G_table = cell(1,D+1);
for i = 1:D+1
    G_table{i} = G(i,modw);
end

or = zeros(2,N);
for d = 0:D
    n = d*(d+1)/2;
    for i = 0:d
        or(:,n+i+1) = [i;d-i];
    end
end

for i = 1:N
    order = sum(or(:,i));
    table = lookup{order+1}{or(2,i)+1};
    for j = 1:length(table.n)
        m = m + (1j)^order*t(i)*G_table{table.n(j)}.*table.coeff(j).*wu(1,:).^table.o(1,j).*wu(2,:).^table.o(2,j);
    end
end
m = amplitude*2*pi*det(Dmat)*exp(-1j*center(:)'*w).*m;