%% MRDataNumerical.m
%
% Function that generates numerically some kspace data given pixelized
% spatial data.
%
% To reduce aliasing artifacts, the resolution of the spatial data should
% be as large as possible.
%
% This code works for non-Cartesian kspace samples thanks to the NUFFT
% routines of Fessler and Sutton. The 'irt' toolbox must be installed.
% SEE: http://www.eecs.umich.edu/~fessler/code/
%
% Matthieu Guerquin-Kern, Biomedical Imaging Group / EPF Lausanne,
% 31-10-2009 (dd-mm-yyyy)

function m = MRDataNumerical(pix_phant,k,FOV)

ref = 1; % refinement
tol = 1e-8;
rs = size(pix_phant);
if norm(round(ref*k(:))-ref*k(:),2)<tol % Cartesian sampling
    %disp('fft');
    X = zeros(ref*rs);
    X(1:rs(1),1:rs(2)) = pix_phant;circshift(X,-floor(rs/2));
    X = fftn(X)/prod(FOV)/prod(rs);
    m = zeros(1,length(k(1,:)));
    for i=1:length(k(1,:))
        m(i) = X(mod(round(-ref*k(1,i)*FOV(1)),ref*rs(1))+1,mod(round(-ref*k(2,i)*FOV(2)),ref*rs(2))+1);
    end
else % for non-Cartesian sampling
    %disp('nufft');
    st = nufft_init(-2*pi*[k(1,:)'/rs(1)*FOV(1),k(2,:)'/rs(2)*FOV(2)], rs, [10,10], 2*rs);
    m = nufft(pix_phant, st)/prod(FOV)/prod(rs);
    m = m.';
end
m = m.*exp(-pi*1j*(k(1,:)+k(2,:)));