%% PixelPhantom.m
%
% Function that pixelizes a 2D analytical phantom.
%
% Matthieu Guerquin-Kern, Biomedical Imaging Group / EPF Lausanne,
% 20-10-2009 (dd-mm-yyyy)

function im = PixelPhantom(phantom,res)

im = zeros(res);
FOV = phantom{1}.FOV; % should be the same for all ellipses that compose the phantom
[r1,r2] = GenerateFullCart2DGrid(res);

for i = 1:length(phantom)
    element = phantom{i};
    switch element.type
        case {'ellipse'}
            theta = element.angle;
            width = element.width;
            center = element.center;
            amplitude = element.amp;
            x1 = r1*FOV(1)/res(1)-center(1);
            x2 = r2*FOV(2)/res(2)-center(2);
            u1 = 2/(width(1))*(cos(theta)*x1+sin(theta)*x2);
            u2 = 2/(width(2))*(-sin(theta)*x1+cos(theta)*x2);
            ind = find(sqrt(u1.^2+u2.^2)<=1);
        otherwise
            error('this kind of phantom is unknown');
    end
    im(ind) = im(ind) + amplitude;
end