%% ShowPhantom.m
%
% Function that plots an image of a 2D analytical phantom.
%
% Matthieu Guerquin-Kern, Biomedical Imaging Group / EPF Lausanne,
% 31-10-2009 (dd-mm-yyyy)

function im = ShowPhantom(phantom,s,res)

FOV = phantom{1}.FOV;

if nargin<3
    res = floor(512*FOV);
end

im = PixelPhantom(phantom,res);
switch numel(s)
    case 0
        % nothing to do assume homogeneous unit sensitivity;
    case 1
        im  = im*s;
    otherwise
        sens_num = zeros(size(im));
        D = floor((sqrt(1+numel(s)*8)-3)/2); % works only in 2D!
        [X1,X2] = GenerateFullCart2DGrid(res);
        R = [X1(:)'*FOV(1)/res(1);X2(:)'*FOV(2)/res(2)];clear X1 X2;
        M = Polynomial2DMatrix(R,D);
        sens_num(:) = M*s;clear M;
        im = im.*sens_num;
end
imagesc(im);colormap gray;colorbar;axis image