% Experiment 1: polynomial approximation of sensitivities
%
% Matthieu Guerquin-Kern, Biomedical Imaging Group / EPF Lausanne,
% 20-10-2009 (dd-mm-yyyy)

clear all;close all;
addpath('misc_sensitivities/');
%make;

%% Phantom
DefineSL;
res = 2^8*[1 1];
im = PixelPhantom(SL,res);

%% Coils simulation
coil.Nb_coils = 12;
coil.res = res;
coil.type = 'biot';
coil.param.FS = 0.28; % FOV width
coil.param.D = 0.15; % Distance center->coil
coil.param.R = 0.05; % radius of the coil
coil.param.rand = 1;
coil = simulate_sensitivities(coil);

%% Coils approximation
D_max = 13;
D_range = 1:1:D_max;

support = (im>1e-3);numel(find(support))
NbCoils = size(coil.sensitivity,3);

tic;
s=cell(1,NbCoils);
nrmse = zeros(length(D_range),NbCoils);
ser = zeros(1,NbCoils);
maxerror = zeros(length(D_range),NbCoils);
SER_mean = zeros(size(D_range));
NRMSE_mean = zeros(size(D_range));
maxerror_mean = zeros(size(D_range));
for indD = 1:length(D_range)
    D = D_range(indD);
    for indCoil = 1:NbCoils
        [s{indCoil},nrmse(indD,indCoil),ser(indD,indCoil),maxerror(indD,indCoil)] = PolynomialSensFitting(coil.sensitivity(:,:,indCoil),D,[1,1],support);
    end
    SER_mean(indD) = mean(ser(indD,:));
    NRMSE_mean(indD) = mean(nrmse(indD,:));
    maxerror_mean(indD) = mean(maxerror(indD,:));
    fprintf('order %d . NRMSE : %g | relative max. error: %g\n',D,NRMSE_mean(indD),maxerror_mean(indD));
end
toc
%figure;stem(D_range,10*log10(SER_mean),'b');xlabel('polynomial degree');ylabel('Signal to Error ratio (dB)')

%%
close all;
figure;plot(D_range,10*log10(ser),'-o');xlabel('polynomial degree');ylabel('Signal to Error ratio (dB)');grid on;axis([1,D_max,0,max(10*log10(ser(:)))]);set(gca,'XTick',D_range);
figure;plot(D_range,nrmse,'-o');xlabel('polynomial degree');ylabel('NRMSE');grid on;axis([1,D_max,0,max(nrmse(:))]);set(gca,'XTick',D_range);
figure;plot(D_range,maxerror,'-o');xlabel('polynomial degree');ylabel('relative max. absolute error');grid on;axis([1,D_max,0,max(maxerror(:))]);set(gca,'XTick',D_range);

%% Export results in LaTeX table

D_str = [];
NRMSE_str = [];
SER_str = [];
max_error_str = [];
dcl_str = '|c|';
for i = 1:length(D_range)
    dcl_str = [dcl_str, 'c|'];
    D_str = [D_str, sprintf(' %d',D_range(i)) ' &'];
    NRMSE_str = [NRMSE_str, sprintf(' %3.2e',NRMSE_mean(i)) ' &'];
    SER_str = [SER_str, sprintf(' %3.1f',10*log10(SER_mean(i))) ' &'];
    max_error_str = [max_error_str , sprintf(' %3.1e',maxerror_mean(i)) ' &'];
end
fexp1 = fopen('../table_exp1.tex','w+');
fprintf(fexp1,'\\begin{center}\n\\begin{tabular}{%s}\n\\hline Degree & %s \\\\\n\\hline\\hline NRMSE & %s \\\\\n\\hline SER (dB) & %s \\\\\n\\hline max.\\ error & %s \\\\\n\\hline\n\\end{tabular}\n\\end{center}',dcl_str,D_str(1:end-1),NRMSE_str(1:end-1),SER_str(1:end-1),max_error_str(1:end-1));
fclose(fexp1);