%% experiment 3: inverse crime example
%
% Matthieu Guerquin-Kern, Biomedical Imaging Group / EPF Lausanne,
% 20-10-2009 (dd-mm-yyyy)

clear all;close all;clc
addpath('misc_sensitivities/');
make;

%% Phantom
define_SheppLogan

%% Coils

res = 2^7*[1 1];
coil.Nb_coils = 4;
coil.res = res;
coil.type = 'homogeneous';%'biot'
coil.param.FS = 0.3; % FOV width
coil.param.D = 0.17; % Distance center->coil
coil.param.R = 0.05; % radius of the coil
coil = simulate_sensitivities(coil);

%% Polynomial approximation of coils
order = 0;
or = [];
for o = 0:order
    for i = 0:o
        or = [or, [i;o-i]];
    end
end
phant = build_phantom_ellipses(phantom,res);
ind = find(phant>1e-3);
[X,Y] = meshgrid((1:res(1))/res(1)*FOV(1),(res(2):-1:1)/res(2)*FOV(2));
R = [X(ind)';Y(ind)'];
M = zeros(length(ind),size(or,2));
for i = 1:size(or,2)
    for j=1:length(ind)
        M(j,i) = prod(R(:,j).^or(:,i));
    end
end

for i = 1:size(coil.sensitivity,3)
    sens = coil.sensitivity(:,:,i);
    [param{i},pint,residue{i},rint,stats] = regress(sens(ind),M);
    %param{i} = M\sens(ind);
    
    sens_approx{i} = zeros(size(sens));
    for j = 1:size(or,2)
        sens_approx{i} = sens_approx{i} + param{i}(j)*X.^or(1,j).*Y.^or(2,j);
    end
    figure;imagesc(abs(sens_approx{i}).*(phant>1e-3));colormap gray;axis image;colorbar;
    SER(i) = sqrt(sens(ind)'*sens(ind))/sqrt(residue{i}'*residue{i});
    subplot(1,size(coil.sensitivity,3),i);imagesc(abs(phant));colormap gray;axis image;colorbar;
    fprintf('coil %d . Signal to error ratio (dB): %g\n max error: %g',i,20*log10(SER(i)),max(abs(residue{i}(:)))/max(abs(sens(ind))));
end

%% K-space trajectory
res_traj = 2^7*[1,1];
% generate k-space trajectory (full Cartesian in that case)
dk = 1./FOV;
[kX,kY] = meshgrid( dk(1)*(-floor(res_traj(1)/2):floor((res_traj(1)-1)/2)),...
    -dk(2)*(-floor(res_traj(2)/2):2:floor((res_traj(1)-1)/2))    );
k_space = [kX(:), kY(:)]';

%% perform analytical computations
t0 = clock();
lookup = [];
m = cell(1,length(param));
m{1} = zeros(1,size(k_space,2));
for j = 1:length(ellipse)
    [val,lookup] = response_ellipse(phantom{j},param{1},k_space,lookup);
    m{1} = m{1} + val;
end
t_ref = etime(clock(),t0);
Rec = ifftshift(reshape(m{1},res_traj(1),res_traj(2)))*prod(res_traj)/prod(FOV);
%figure(2);subplot(1,length(param),i);imagesc(fftshift(log10(abs(Rec))));%title('computations in fourier');axis image;colorbar;
rec = ifftn(Rec);
figure(5);imagesc(imag(rec));colormap gray;axis image;colorbar;title('reconstruction in space');drawnow;